//-- Copyright Drew Bagnell, 1998


//Experimental 16 servo motor driver with
//high precision control (within 2 degrees)


//One potentially useful improvement to make sure you only call servo changes when nothing
//is happening to the servo... (only in the first 1000 e clocks after a switch.) Note,
//the walk program does this by checking for a change in clock_counter, which happens
//immediately after the ISR.
/**************************** Includes **********************************/
#include <hc11.h>
#include <mil.h>

/************************************************************************/

/*************************** Prototypes *********************************/
#pragma interrupt_handler servo_handON
#pragma interrupt_handler servo_handB
#pragma interrupt_handler servo_handC

void init_servos(void);
void servo(int, int);
void servo_handB();
void servo_handC();
/************************************************************************/


/***************************** Globals **********************************/
unsigned int ontimeB[8];
unsigned int ontimeC[8];

unsigned char indexB,indexC;
unsigned char servomaskB[8],servomaskC[8];

int clock_counter;
/************************************************************************/



void 
init_servos(void)
{

  int i;
  INTR_OFF();
  CLEAR_BIT(TCTL1,0x07);     //interrupt will affect no pins in port A

  for(i = 0 ; i < 8 ; i++)
  {
    ontimeB[i] = 3000;
    ontimeC[i] = 3000;

    servomaskB[i] =  1 << i;
    servomaskC[i] =  1 << i;
  }

  TOC1 = 0; //turn them all on at the rollover
  TOC2 = TOC3 = 3000; //start turning off at mid-position

  SET_BIT(TMSK1,0xE0);            /* Enable OC1,2,3 interrupt */

  DDRC = 0xff;

  clock_counter = 0;
  INTR_ON();
}

/*
   Applies or removes on from servo by turning the servomask
   to zero. In particular, if state= 0 , servo is turned off
   and state=1 turns it back on.
*/
void
servo_power(int offset, int state)
{
	   
        if (offset < 8 )
        {
        servomaskB[offset] = state << offset;
        }
        else if (offset < 16 )
        {
        servomaskC[offset] = state << offset;
        }
}


void
servo(int offset, int newwidth)
{
  int i;
  int deltaWidth;

  if ( offset < 8 )
  {
  deltaWidth = newwidth - ontimeB[offset];
  ontimeB[offset] += deltaWidth;
  }
  else if ( offset < 16 )
  {
  deltaWidth = newwidth - ontimeC[offset-8];
  ontimeC[(offset)-8] += deltaWidth;
  }  


}

void
servo_handON()
{
        PORTC = *(servomaskC+indexC);
        PORTB = *(servomaskB+indexB);
        TOC1 += 5000;
        CLEAR_FLAG(TFLG1,0x80);          /* Clear OC1 flag */
        indexB = indexC = (indexC + 1)&0x07;
        clock_counter++;
}



void 
servo_handB()
{
    PORTB = 0;
    CLEAR_FLAG(TFLG1,0x40);          /* Clear OC2 flag */
    INTR_ON();
    TOC2 += *(ontimeB+indexB) + (TOC1 - TOC2);
}

void 
servo_handC()
{
    PORTC = 0;
    CLEAR_FLAG(TFLG1,0x20);          /* Clear OC3 flag */
    INTR_ON();
    TOC3 += *(ontimeC+indexC) + (TOC1 - TOC3);
}


