/************************************************************************
 * 			MEKATRONIX Copyright 1998 		        *
 * Title        avoidtj.c                                               *
 * Programmer	Keith L. Doty	        				*
 * Date         March 13, 1998                                          *
 * Version	1			                		*
 *								        *
 * Description								*
 *    A very simple collision avoidance program. TJ will read 		*
 *    each IR detectors, and turn away from any obstacles in its	*
 *    path. Also, if something hits TJ's bumper, it will back up,	*
 *    turn, and go on.							*
 *			                                                *     					*
 ************************************************************************/
 
/**************************** Includes **********************************/
#include <tjbase.h>
/************************ End of includes *******************************/

/**************************** Constants *********************************/
#define FORWL  2000
#define FORWR  4000
#define BACKL  4000
#define BACKR  2000
/************************ End of Constants ****************************/

/*************************** Prototypes *********************************/
void turn(void);
/************************ End of Prototypes *****************************/

/***************************** Globals **********************************/
int rspeed, lspeed;
/************************** End of Globals ******************************/


void main(void)
/****************************** Main ***********************************/
{
  int irdr, irdl;
  int i;

  init_motortj();
  init_analog();
  init_ir();
  
  DDRC = 0x00;           /* Set PORTC (bumper) for input */

  while(1)
  {

/* The following block will read the IR Detector (ird) ports, and decide */
/* whether TJ needs to turn to avoid any obstacles                       */
    irdr = ir_value(6);
    irdl = ir_value(7);

    if (irdr > 100)
      rspeed = BACKR;
    else
      rspeed = FORWR;
    if (irdl > 100)
      lspeed = BACKL;
    else
      lspeed = FORWL;

    motortj(RIGHT_MOTOR, rspeed); 
    motortj(LEFT_MOTOR, lspeed);

/* This "if" statement checks the bumper. If the bumper is pressed, */
/* Tj will back up, and turn.                                       */

    if(PORTC & 0x20)
    {
      motortj(LEFT_MOTOR, BACKL);
      motortj(RIGHT_MOTOR, BACKR);
      for(i = 0; i < 25000; i++);
      turn();
    }

/* T= 27*(i-1)+24 cycles, i=2500, cycle = 0.5 microseconds */ 
  for(i = 1; i < 2500; i++);   /* T = 33748.5 microseconds */
  }    
}
/**************************** End of Main ******************************/

void turn(void)
/************************************************************************
 * Function:  Will turn in a random direction for a random amount of    *
 * time                                                                 *
 * Returns:  None                                                       *
 *                                                                      *
 * Inputs                                                               *
 *   Parameters: None                                                   *
 *   Globals:    None                                                   *
 *   Registers:  TCNT                                                   *
 * Outputs                                                              *
 *   Parameters: None                                                   *
 *   Globals:    None                                                   *
 *   Registers:  None                                                   *
 * Functions called: None                                               *
 * Notes:                                                               *
 ************************************************************************/
{
 int i;
 unsigned rand;

 rand = TCNT;

 if (rand & 0x0001)
 {
   motortj(RIGHT_MOTOR, FORWR);
   motortj(LEFT_MOTOR, BACKL);
 }
 else
 {
   motortj(RIGHT_MOTOR, BACKR);
   motortj(LEFT_MOTOR, FORWL);
 }
 for (i = 0; i < rand; i++);
}


